/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com
*/

/** \file wpan.h
    \brief Wireless Personal Area Network routines


Put this in your config.h
\code
// ------------------------
// WPAN (802.15.4) wireless
// ------------------------

#define WPAN_USE_MRF24J50
// ------------------------
\endcode
*/

#ifndef __wpan_h
#define __wpan_h



#include "pic_utils.h"
#include "config.h"
#include "mrf24j40.h"
#include "spi_hw.h"


extern uns8 pkt_received;

#define WPAN_ADDR_TYPE_EXTENDED	0b00000011
#define WPAN_ADDR_TYPE_SHORT    0b00000010
#define WPAN_ADDR_TYPE_NONE   	0b00000000

#define MAC_CMD_ASSOC_REQ		0x01
#define MAC_CMD_ASSOC_RES		0x02
#define MAC_CMD_DISASSOC		0x03
#define MAC_CMD_DATA_REQ		0x04
#define MAC_CMD_PAN_ID_CONFLICT	0x05
#define MAC_CMD_ORPHAN			0x06
#define MAC_CMD_BEACON_REQ		0x07
#define MAC_CMD_COORD_REALIGN	0x08
#define MAC_CMD_GTS_REQ			0x09

#define FRAME_TYPE_BEACON		0b000
#define FRAME_TYPE_DATA			0b001
#define FRAME_TYPE_ACK			0b010
#define FRAME_TYPE_MAC_COMMAND	0b011



typedef struct _wpan_address {
	uns8 source_address_type; //	0 - no address / 2 - short / 3 - long
	uns16 source_pan_id;
	uns16 source_sa;
	uns8 source_ea[8];

	uns8 dest_address_type; //	0 - no address / 2 - short / 3 - long
	uns16 dest_pan_id;
	uns16 dest_sa;
	uns8 dest_ea[8];
} wpan_address;


/** 
	\brief Setup IO as requred 
	
	Set up ports and pins for WPAN use
*/
void wpan_setup_io();

/**
	\brief Initialise this and lower layers ready for use.
	
	Initialise underlying hardware (typically mrf24j40)
*/
void wpan_init();

/**

	\brief Process WPAN layer
	
	Call wpan_process() regularly to handle events at the WPAN layer. Typically this
	includes handling any received packets, processing them, and handing up to a higher
	layer.
	
*/
void wpan_process();

/**

	\brief Debug routine that prints out the mac command type
	
	Print the MAC layer command details
*/	
void wpan_print_mac_command(uns8 *data);

/**

	\brief Print information about the WPAN frame type
*/
void wpan_print_frame_type(uns8 frame_type);

/**
	\brief Print the address of the packet
	
	Print out the source and destination address
*/
void wpan_print_address(wpan_address *addr);

/**
	\brief Callback for data received
	
	After the packet has been processed and the address extracted, it is delivered
	to the callback function.
	
*/
void wpan_data_received_callback(wpan_address *addr, uns8 *data, uns8 data_size, uns8 lqi, uns8 rssi);

/**
	\brief Callback for data transmitted
	
	Once the lower layers have attempted to transmit the packet, the results will
	be presented to the callback.
	
	\param status Set to 0 for success or 1 for failure
	\param retries Set to the number of retries
	\param channel_busy Set to 1 if failure was due to the channel being busy.

*/
void wpan_data_transmitted_callback(uns8 status, uns8 retries, uns8 channel_busy);

/** 
	\brief Handle reception of packet
	
	Internal routine for WPAN layer to handle the reception of a packet.
*/
void wpan_handle_receive();


#endif
